/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.ui;

import org.soafaces.bundle.client.rpc.FetchJSONCallback;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.RootPanel;

/**
 * <p>
 * A convenient subclass of Weblet. Serializes the input JavaBean associated
 * with the Weblet using JSON and presents it to the Weblet GWT widget
 * as a <code>JSONObject</code>
 * </p>
 * <p>
 * This class should be subclassed to create a custom editor (Weblet) for a
 * given Tasklet or to create a standalone application. This UI is considered
 * a "design time" UI for customizing the Tasklets input JavaBean.
 * </p>
 * 
 * @author Sam Taha
 */
public abstract class SimpleJSONWeblet extends Weblet {
  private boolean uiLoaded=false;

  /**
   * Convenient implementation to return the top level UI container.
   * @return
   */
  public Panel getMainPanel() { return RootPanel.get(); }
  
  /**
   * Convenient implementation that pulls the input JavaBean from the server
   * and wrapps it in a JSONObject.
   */
  public final void buildUI()
  {
    if(uiLoaded)
      return;
    uiLoaded=true;

    FetchJSONCallback callback = new FetchJSONCallback () {
      
      public void onFetchJSONSuccess(JSONObject jsonBean)
      {
        buildUI(jsonBean);
      } 
      
      public void onFailure(Throwable ex)
      {
        Window.alert("Error: Problem processing server request: " + ex.getMessage());
      }
    };
    
    getContext().fetchInputBean(callback);
  }
  
  /**
   * Method to implement in subclass. Use this method to populate the UI and
   * setup action handlers and such.
   * 
   * @param jsonBean Input JavaBean serialized to JSON.
   */
  protected abstract void buildUI(JSONObject jsonBean);
}
