/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.services.client.rpc;

import com.google.gwt.json.client.JSONParser;
import com.google.gwt.json.client.JSONValue;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * <p>
 * Callback used during <code>UniversalClient</code> request processing.
 * Result will be returned in JSON format.
 * </p>
 * <p>
 * The posssible <code>onFailure()</code> exceptions that can be returned
 *  if there is a problem include:
 * <ul>
 *  <li><code>UniveralClientRemoteException</code> when there is a server-side problem
 *    processing the request.</li>
 *  <li>Standard RPC exceptions that can be thrown by GWT.</li>
 * </ul>
 * </p>
 * 
 * @author Sam Taha
 */
public abstract class UniversalClientJSONCallback implements AsyncCallback {
  
  public UniversalClientJSONCallback() {}
  
  /**
   * Implement this method to process response from RPC. Result must be
   * a JSON string.
   * 
   * @param result
   */
  public abstract void onUniversalClientSuccess(JSONValue result);

  final public void onSuccess(Object result)
  {
    JSONValue jsonObj = null;
    if(result != null)
    {
      try {
        GenericDataWrapper proxy = (GenericDataWrapper) result;
        if(proxy._Json == null)
          return;
        jsonObj = JSONParser.parse(proxy._Json);
      }catch(Throwable ex) {
        Window.alert("Problem parsing JSON string: " + ex.getMessage());
        return;
      }
    }

    onUniversalClientSuccess(jsonObj);
  }
}
